#ifndef WIN32
#define unix
#endif

#define MAIN
#include "sisocks.h"

#ifndef unix
typedef int socklen_t;
#define CAN_TCP_NODELAY
#endif

#ifdef unix
#include <sys/un.h> /* needed for unix sockets */
#endif

#define IPC_CMD_HELO 0x00000100
#define IPC_CMD_OLEH 0x00010000
#define IPC_CMD_HELO 0x00000100

#define IPC_CMD_ACCEPT_FORMAT 0x01
#define IPC_CMD_DUMP_FRAME    0x02
#define IPC_CMD_CLOSE         0x03
#define IPC_CMD_BEGIN         0x04

#define IPC_RES_YES  0x00010100
#define IPC_RES_NO   0x00020200

static SOCKET s = -1;

static int ipc_connect(const char *host, int port) {
#ifdef unix
  struct sockaddr_un sau;
#endif
  SAIN sai;
  int i;

  if (port) {
    memset(&sai,0,sizeof(sai));
    build_sin(&sai,host,port);
  } else {
#ifdef unix
    memset(&sau,0,sizeof(sau));
    sau.sun_family=AF_LOCAL;
    strcpy(sau.sun_path,host); // FIXME: possible overflow!
#else
    return -11;  // unsupported
#endif
  }
  s = socket(port?AF_INET:AF_LOCAL, SOCK_STREAM, 0);
  if (port) {
#ifdef CAN_TCP_NODELAY
    int opt=1;
    setsockopt(s, IPPROTO_TCP, TCP_NODELAY, (const char*) &opt, sizeof(opt));
#endif
    i=connect(s, (SA*)&sai, sizeof(sai));
  }
#ifdef unix
  else
    i=connect(s, (SA*)&sau, sizeof(sau));
#endif
  if (i==-1) {
    closesocket(s); s=-1;
    return -1; // connect failed
  }
  return 0;
}

static void ipc_close() {
  if (s>-1) {
    closesocket(s);
    s=-1;
  }
}

struct s_header {
  int cmd;
  int len;
};

static int ipc_send_hdr(int cmd, int len) {
  if (s<0) return -1;
  {
    struct s_header h = { cmd, len };
    if (send(s, (char*)&h, 8, 0) != 8) { ipc_close(); return -2; }
  }
  return 0;
}

static int ipc_send_data(char *buf, int len) {
  if (s<0) return -1;
  if (len>0) {
    if (send(s, (char*) buf, len, 0) != len) { ipc_close(); return -2; }
  }
  return 0;
}

static int ipc_send(int cmd, void *buf, int len) {
  int i = ipc_send_hdr(cmd, len);
  if (i==0 && len>0) i=ipc_send_data(buf, len);
  return i;
}

static int ipc_resp() {
  int i, n;
  if (s<0) return -1;
  if (recv(s, (void*)&i, 4, 0)!=4) { ipc_close(); return -1; }
  return i;
} 

/* mplayer callbacks */

int vo_accept_format(unsigned int format) {
  if (s<0) ipc_connect("127.0.0.1",7922);
  if (ipc_send(IPC_CMD_ACCEPT_FORMAT, &format, 4)) return 0;
  return (ipc_resp()==IPC_RES_YES)?1:0;
}

struct s_frame {
  int w, h, f, chs;
};

void vo_dump_frame(void *buf, int w, int h, int f, int chs, int flags) {
  struct s_frame fr = { w, h, f, chs };
  int l = sizeof(fr) + w*h*4;
  ipc_send_hdr(IPC_CMD_DUMP_FRAME, l);
  ipc_send_data((void*)&fr, sizeof(fr));
  ipc_send_data(buf, l-sizeof(fr));
}

void vo_begin(int w, int h, int fmt) {
  int a[3] = { w, h, fmt };
  ipc_send(IPC_CMD_BEGIN, &a, 3*4);
}

void vo_end() {
  ipc_send(IPC_CMD_CLOSE, 0, 0);
  ipc_close();
}
